//
//  YHInformationFillVC.swift
//  galaxy
//
//  Created by edy on 2024/4/11.
//  Copyright © 2024 https://www.galaxy-immi.com. All rights reserved.
//

import UIKit

class YHMessageDetailListVC: YHBaseViewController {
    
    var type:Int = -1
    var msgArr:[YHDetailMessageModel] = []
    // 滑动过程中出现在屏幕上的未读消息集合
    var visiblemsgIdSet = Set<String>()
    var visibleMsgArr = [YHDetailMessageModel]()
    
    lazy var viewModel = {
        let model = YHMsgViewModel()
        return model
    }()
    
    lazy var navBar: YHCustomNavigationBar = {
        let bar = YHCustomNavigationBar.navBar()
        bar.backgroundColor = .white
        if let title = self.title, !title.isEmpty {
            bar.title = title
        }
        bar.showRightButtonType(.clean)
        bar.backBlock = {
            [weak self] in
            guard let self = self else { return }
            self.navigationController?.popViewController(animated: true)
        }
        bar.rightBtnClick = {
            [weak self] in
            guard let self = self else { return }
            YHCommonAlertView.show("清除未读", "确定要清除所有未读提示吗？", "取消", "确认") {
                self.markAllMsgsRead()
            }
        }
        return bar
    }()
    
    lazy var tableView: UITableView = {
        let tableView = UITableView(frame:.zero, style:.grouped)
        tableView.estimatedSectionHeaderHeight = 1.0
        tableView.estimatedSectionFooterHeight = 1.0
        tableView.showsVerticalScrollIndicator = false
        tableView.separatorStyle = .none
        tableView.delegate = self
        tableView.dataSource = self
        tableView.backgroundColor = UIColor(hex:0xF8F8F8)
        tableView.backgroundView = emptyDataTipsView
        tableView.register(UITableViewCell.self, forCellReuseIdentifier: "UITableViewCell")
        tableView.register(YHDetailMessageCell.self, forCellReuseIdentifier: YHDetailMessageCell.cellReuseIdentifier)
        return tableView
    }()
    
    lazy var emptyDataTipsView:UIView = {
        let view = UIView(frame: CGRect(x: 0, y: 0, width: KScreenWidth, height: 124))
        view.isHidden = true
        let imgView = UIImageView(image: UIImage(named: "msg_no_data"))
        view.addSubview(imgView)
        
        let label = UILabel()
        label.textColor = UIColor(hex: 0x94A3B8)
        label.font = UIFont.PFSC_R(ofSize: 14)
        label.textAlignment = .center
        label.text = "暂无新消息".local
        view.addSubview(label)
        
        imgView.snp.makeConstraints { make in
            make.top.equalToSuperview().offset(204)
            make.width.height.equalTo(92)
            make.centerX.equalTo(view)
        }
        
        label.snp.makeConstraints { make in
            make.top.equalTo(imgView.snp.bottom).offset(12)
            make.centerX.equalTo(view)
        }
        return view
    }()

    override func viewDidLoad() {
        super.viewDidLoad()
        setupUI()
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        getMsgList()
    }
    
    func setupUI() {
        
        gk_navigationBar.isHidden = true
        view.backgroundColor = .white
        view.addSubview(tableView)
        view.addSubview(navBar)
        navBar.snp.makeConstraints { make in
            make.left.right.equalToSuperview()
            make.top.equalToSuperview().offset(k_Height_statusBar())
            make.height.equalTo(k_Height_NavContentBar)
        }
        tableView.snp.makeConstraints { make in
            make.left.right.equalToSuperview()
            make.top.equalToSuperview().offset(k_Height_NavigationtBarAndStatuBar)
            make.bottom.equalToSuperview()
        }
    }
    
    func getMsgList() {
        YHHUD.show(.progress(message: "加载中..."))
        self.viewModel.getMsgDetailList(type: self.type) {
            [weak self] success, error in
            YHHUD.hide()
            guard let self = self else { return }
            msgArr.removeAll()
            msgArr.append(contentsOf: self.viewModel.msgDetailList)
            emptyDataTipsView.isHidden = msgArr.count > 0
            self.tableView.reloadData {
                [weak self] in
                guard let self = self else { return }
                self.addUpAppearUnreadMessages()
                self.markAppearMessagsReaded()
            }
        }
    }
    
    func markAllMsgsRead() {
        YHHUD.show(.progress(message: "清除中..."))
        self.viewModel.readMessages(type:self.type, msgIds:nil, isAllRead:true) {
            [weak self] success, error in
            YHHUD.hide()
            guard let self = self else { return }
            if success {
                YHHUD.flash(message: "清除成功")
                for item in self.msgArr {
                    item.isRead = true
                }
                self.tableView.reloadData()
                return
            }
            var msg = "清除失败"
            if let error = error {
                if error.errorMsg.count > 0 {
                    msg = error.errorMsg
                }
            }
            YHHUD.flash(message: msg)
        }
    }
    
    func markAppearMessagsReaded() {
        
            var msgIds:[Int] = []
            var resultArr:[YHDetailMessageModel] = []
            for msgItem in visibleMsgArr {
                if let msgId = Int(msgItem.id), msgItem.isRead == false  {
                    msgIds.append(msgId)
                    resultArr.append(msgItem)
                }
            }
            if msgIds.count <= 0 { return }
            self.viewModel.readMessages(type: self.type, msgIds: msgIds, isAllRead:false) {
                [weak self] success, error in
                guard let self = self else { return }
                if success {
                    for msgItem in resultArr {
                        msgItem.isRead = true
                    }
                    self.tableView.reloadData()
                    return
                }
//                var msg = "标记已读失败"
//                if let error = error {
//                    if error.errorMsg.count > 0 {
//                        msg = error.errorMsg
//                    }
//                }
//                YHHUD.flash(message: msg)
            }
        }
}

extension YHMessageDetailListVC: UITableViewDelegate, UITableViewDataSource {
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return msgArr.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        
        let cell = tableView.dequeueReusableCell(withIdentifier: YHDetailMessageCell.cellReuseIdentifier, for: indexPath) as! YHDetailMessageCell
        if 0 <= indexPath.row && indexPath.row < msgArr.count {
            let item = msgArr[indexPath.row]
            cell.updateModel(item)
        }
        return cell
    }
    
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return UITableView.automaticDimension
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
       
    }

    private func tableView(_ tableView: UITableView, viewForHeaderInSection section: Int) -> CGFloat {
        return 1.0
    }
    
    private func tableView(_ tableView: UITableView, viewForFooterInSection section: Int) -> CGFloat {
        return 1.0
    }
    
    func tableView(_ tableView: UITableView, viewForFooterInSection section: Int) -> UIView? {
        
        return UIView()
    }
    
    func tableView(_ tableView: UITableView, viewForHeaderInSection section: Int) -> UIView? {

        return UIView()
    }
    
    func scrollViewDidEndDragging(_ scrollView: UIScrollView, willDecelerate decelerate: Bool) {
        if scrollView == tableView && !decelerate {
            // 在这里处理滑动停止事件
            print("TableView 滑动停止")
            markAppearMessagsReaded()
        }
    }
    
    func scrollViewDidScroll(_ scrollView: UIScrollView) {
        addUpAppearUnreadMessages()
    }
    
    // 累计出现在屏幕上的未读消息集合
    func addUpAppearUnreadMessages() {
        guard let visibleIndexPaths = tableView.indexPathsForVisibleRows else { return }
        for indexPath in visibleIndexPaths {
            if 0 <= indexPath.row && indexPath.row < msgArr.count {
                let msgItem = msgArr[indexPath.row]
                if !msgItem.isRead {
                    if !visiblemsgIdSet.contains(msgItem.id) {
                        visiblemsgIdSet.insert(msgItem.id)
                        visibleMsgArr.append(msgItem)
                    }
                }
            }
        }
    }
}

